/*
 * Controls.java
 *
 * Created on April 3, 2006, 4:18 PM
 */

package pl.edu.agh.ssm2006.jpeg2000demo;

import java.awt.image.BufferedImage;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyVetoException;
import java.beans.VetoableChangeListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.imageio.stream.FileImageInputStream;
import javax.imageio.stream.MemoryCacheImageInputStream;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileFilter;

import org.apache.commons.io.filefilter.DirectoryFileFilter;
import org.apache.commons.io.filefilter.SuffixFileFilter;

/**
 * 
 * @author SG0893872
 */
public class ControllInternalFrame extends javax.swing.JInternalFrame {
	public static final String TOOL_TIPS_BUNDLE = "pl/edu/agh/ssm2006/jpeg2000demo/ToolTipTexts";

	public javax.swing.JFileChooser jFileChooser = new JFileChooser();

	/**
	 * 
	 */
	private static final long serialVersionUID = 6074926440214103788L;

	private Main main;

	/** Creates new form Controls */
	public ControllInternalFrame(String label, Main m) {
		super(label, true, true, true, false);
		this.main = m;
		addVetoableChangeListener(new VetoableChangeListener() {
			public void vetoableChange(PropertyChangeEvent e)
					throws PropertyVetoException {
				if (e.getPropertyName().equals(IS_CLOSED_PROPERTY)
						&& (((Boolean) e.getOldValue()).booleanValue() == false)) {
					main.getControlsMenuItem().setSelected(false);
					setVisible(false);
					throw new PropertyVetoException(IS_CLOSED_PROPERTY, e);
				}
			}

		});

		// file chooser starts from current directory
		File dir = new File(".");
		jFileChooser.setCurrentDirectory(dir);
		// FileFilter
		final SuffixFileFilter jpgImages = new SuffixFileFilter(new String[] {
				".jpg", ".jpeg", ".JPG", ".JPEG", ".Jpg", ".Jpeg" });
		final SuffixFileFilter pngImages = new SuffixFileFilter(new String[] {
				".png", ".PNG", ".Png" });
		final SuffixFileFilter bmpImages = new SuffixFileFilter(new String[] {
				".bmp", ".BMP", ".Bmp" });
		final SuffixFileFilter gifImages = new SuffixFileFilter(new String[] {
				".gif", ".GIF", ".Gif" });

		FileFilter allImagesFilter = new FileFilter() {

			public boolean accept(File pathname) {
				return jpgImages.accept(pathname) || pngImages.accept(pathname)
						|| bmpImages.accept(pathname)
						|| gifImages.accept(pathname)
						|| DirectoryFileFilter.INSTANCE.accept(pathname);
			}

			@Override
			public String getDescription() {
				return "All image types";
			}

		};

		jFileChooser.addChoosableFileFilter(allImagesFilter);

		jFileChooser.addChoosableFileFilter(new FileFilter() {

			public boolean accept(File pathname) {
				return jpgImages.accept(pathname)
						|| DirectoryFileFilter.INSTANCE.accept(pathname);
			}

			@Override
			public String getDescription() {
				return "JPG images";
			}

		});

		jFileChooser.addChoosableFileFilter(new FileFilter() {

			public boolean accept(File pathname) {
				return pngImages.accept(pathname)
						|| DirectoryFileFilter.INSTANCE.accept(pathname);
			}

			@Override
			public String getDescription() {
				return "PNG images";
			}

		});

		jFileChooser.addChoosableFileFilter(new FileFilter() {

			public boolean accept(File pathname) {
				return bmpImages.accept(pathname)
						|| DirectoryFileFilter.INSTANCE.accept(pathname);
			}

			@Override
			public String getDescription() {
				return "BMP images";
			}

		});

		jFileChooser.addChoosableFileFilter(new FileFilter() {

			public boolean accept(File pathname) {
				return gifImages.accept(pathname)
						|| DirectoryFileFilter.INSTANCE.accept(pathname);
			}

			@Override
			public String getDescription() {
				return "GIF images";
			}

		});

		jFileChooser.setFileFilter(allImagesFilter);

		// GUI components initialization
		initComponents();
	}

	public void disableRefreshButton() {
		refreshButton.setEnabled(false);
	}

	public void enableRefreshButton() {
		refreshButton.setEnabled(true);
	}

	private void initComponents() {
		java.awt.GridBagConstraints gridBagConstraints;

		refreshButton = new javax.swing.JButton();
		jPanel1 = new javax.swing.JPanel();
		jLabel6 = new javax.swing.JLabel();
		modeCombo = new javax.swing.JComboBox();
		jLabel7 = new javax.swing.JLabel();
		imageTypeCombo = new javax.swing.JComboBox();
		jLabel8 = new javax.swing.JLabel();
		internalImageCombo = new javax.swing.JComboBox();
		jLabel9 = new javax.swing.JLabel();
		externalImageField = new javax.swing.JTextField();
		externalImageButton = new javax.swing.JButton();
		jLabel3 = new javax.swing.JLabel();
		rescaleCombo = new javax.swing.JComboBox();
		jPanel2 = new javax.swing.JPanel();
		jLabel4 = new javax.swing.JLabel();
		decompositionLevelsCombo = new javax.swing.JComboBox();
		jPanel3 = new javax.swing.JPanel();
		jLabel10 = new javax.swing.JLabel();
		guardBitsCombo = new javax.swing.JComboBox();
		jLabel5 = new javax.swing.JLabel();
		quantizationStepCombo = new javax.swing.JTextField();
		jLabel11 = new javax.swing.JLabel();
		quantizationTypeCombo = new javax.swing.JComboBox();
		jPanel4 = new javax.swing.JPanel();
		jLabel2 = new javax.swing.JLabel();
		bitRateField = new javax.swing.JTextField();
		jLabel1 = new javax.swing.JLabel();
		progressionTypeCombo = new javax.swing.JComboBox();

		getContentPane().setLayout(new java.awt.GridBagLayout());

		refreshButton.setText("Refresh");
		refreshButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				refresh(evt);
			}
		});

		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 4;
		gridBagConstraints.insets = new java.awt.Insets(18, 0, 0, 0);
		getContentPane().add(refreshButton, gridBagConstraints);

		jPanel1.setLayout(new java.awt.GridBagLayout());

		jPanel1.setBorder(javax.swing.BorderFactory
				.createTitledBorder("General"));
		jLabel6.setText("Compression mode: ");
		jLabel6.setToolTipText(java.util.ResourceBundle.getBundle(
				TOOL_TIPS_BUNDLE).getString("mode"));
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
		jPanel1.add(jLabel6, gridBagConstraints);

		modeCombo.setModel(new javax.swing.DefaultComboBoxModel(new String[] {
				"Lossy", "Lossless" }));
		modeCombo.setToolTipText(java.util.ResourceBundle.getBundle(
				TOOL_TIPS_BUNDLE).getString("mode"));
		modeCombo.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				modeChanged(evt);
			}
		});

		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridwidth = 2;
		gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
		gridBagConstraints.weightx = 1.0;
		jPanel1.add(modeCombo, gridBagConstraints);

		jLabel7.setText("Image source: ");
		jLabel7.setToolTipText(java.util.ResourceBundle.getBundle(
				TOOL_TIPS_BUNDLE).getString("imageSource"));
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 1;
		gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
		jPanel1.add(jLabel7, gridBagConstraints);

		imageTypeCombo.setModel(new javax.swing.DefaultComboBoxModel(
				new String[] { "Built-in images", "External images" }));
		imageTypeCombo.setToolTipText(java.util.ResourceBundle.getBundle(
				TOOL_TIPS_BUNDLE).getString("imageSource"));
		imageTypeCombo.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				imageSourceChanged(evt);
			}
		});

		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 1;
		gridBagConstraints.gridy = 1;
		gridBagConstraints.gridwidth = 2;
		gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
		gridBagConstraints.weightx = 1.0;
		jPanel1.add(imageTypeCombo, gridBagConstraints);

		jLabel8.setText("Internal image: ");
		jLabel8.setToolTipText(java.util.ResourceBundle
				.getBundle(TOOL_TIPS_BUNDLE).getString("internalImage"));
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 2;
		gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
		jPanel1.add(jLabel8, gridBagConstraints);

		internalImageCombo.setModel(new javax.swing.DefaultComboBoxModel(
				new String[] { "Clock", "Viper", "Castle", "St_Mary_church",
						"Mountains1", "Mountains2", "Mountains3", "Mountains4",
						"Winter" }));
		internalImageCombo.setToolTipText(java.util.ResourceBundle
				.getBundle(TOOL_TIPS_BUNDLE).getString("internalImage"));
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 1;
		gridBagConstraints.gridy = 2;
		gridBagConstraints.gridwidth = 2;
		gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
		gridBagConstraints.weightx = 1.0;
		jPanel1.add(internalImageCombo, gridBagConstraints);

		jLabel9.setText("External image: ");
		jLabel9.setToolTipText(java.util.ResourceBundle
				.getBundle(TOOL_TIPS_BUNDLE).getString("externalImage"));
		jLabel9.setEnabled(false);
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 3;
		gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
		jPanel1.add(jLabel9, gridBagConstraints);

		externalImageField.setEnabled(false);
		externalImageField.setToolTipText(java.util.ResourceBundle
				.getBundle(TOOL_TIPS_BUNDLE).getString("externalImage"));
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 1;
		gridBagConstraints.gridy = 3;
		gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
		gridBagConstraints.weightx = 1.0;
		jPanel1.add(externalImageField, gridBagConstraints);

		externalImageButton.setText("Select");
		externalImageButton.setEnabled(false);
		externalImageButton
				.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(java.awt.event.ActionEvent evt) {
						openExternalFile(evt);
					}
				});
		externalImageButton.setToolTipText(java.util.ResourceBundle
				.getBundle(TOOL_TIPS_BUNDLE).getString("externalImage"));

		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 2;
		gridBagConstraints.gridy = 3;
		jPanel1.add(externalImageButton, gridBagConstraints);

		jLabel3.setText("Display mode: ");
		jLabel3.setToolTipText(java.util.ResourceBundle.getBundle(
				TOOL_TIPS_BUNDLE).getString("displayMode"));
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 4;
		gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
		jPanel1.add(jLabel3, gridBagConstraints);

		rescaleCombo.setModel(new javax.swing.DefaultComboBoxModel(
				new String[] { "Original", "Rescale palette" }));
		rescaleCombo.setToolTipText(java.util.ResourceBundle.getBundle(
				TOOL_TIPS_BUNDLE).getString("displayMode"));
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 1;
		gridBagConstraints.gridy = 4;
		gridBagConstraints.gridwidth = 2;
		gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
		gridBagConstraints.weightx = 1.0;
		jPanel1.add(rescaleCombo, gridBagConstraints);

		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
		gridBagConstraints.weightx = 1.0;
		getContentPane().add(jPanel1, gridBagConstraints);

		jPanel2.setLayout(new java.awt.GridBagLayout());

		jPanel2.setBorder(javax.swing.BorderFactory
				.createTitledBorder("Wavelet"));
		jLabel4.setText("Decomposition levels: ");
		jLabel4.setToolTipText(java.util.ResourceBundle.getBundle(
				TOOL_TIPS_BUNDLE).getString("decompositionLevels"));
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 0;
		gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
		jPanel2.add(jLabel4, gridBagConstraints);

		decompositionLevelsCombo.setModel(new javax.swing.DefaultComboBoxModel(
				new String[] { "0", "1", "2", "3", "4", "5", "6", "7", "8",
						"9", "10" }));
		decompositionLevelsCombo.setSelectedIndex(5);
		decompositionLevelsCombo.setToolTipText(java.util.ResourceBundle
				.getBundle(TOOL_TIPS_BUNDLE).getString("decompositionLevels"));
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 1;
		gridBagConstraints.gridy = 0;
		gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
		gridBagConstraints.weightx = 1.0;
		jPanel2.add(decompositionLevelsCombo, gridBagConstraints);

		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 1;
		gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
		gridBagConstraints.weightx = 1.0;
		getContentPane().add(jPanel2, gridBagConstraints);

		jPanel3.setLayout(new java.awt.GridBagLayout());

		jPanel3.setBorder(javax.swing.BorderFactory
				.createTitledBorder("Quantization"));
		jLabel10.setText("Guard bits: ");
		jLabel10.setToolTipText(java.util.ResourceBundle.getBundle(
				TOOL_TIPS_BUNDLE).getString("guardBits"));
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
		jPanel3.add(jLabel10, gridBagConstraints);

		guardBitsCombo
				.setModel(new javax.swing.DefaultComboBoxModel(new String[] {
						"1", "2", "3", "4", "5", "6", "7" }));
		guardBitsCombo.setSelectedIndex(2);
		guardBitsCombo.setToolTipText(java.util.ResourceBundle.getBundle(
				TOOL_TIPS_BUNDLE).getString("guardBits"));
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
		gridBagConstraints.weightx = 1.0;
		jPanel3.add(guardBitsCombo, gridBagConstraints);

		jLabel5.setText("Quantisation step: ");
		jLabel5.setToolTipText(java.util.ResourceBundle.getBundle(
				TOOL_TIPS_BUNDLE).getString("quantizationStep"));
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 2;
		gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
		jPanel3.add(jLabel5, gridBagConstraints);

		quantizationStepCombo.setText("0.0078125");
		quantizationStepCombo.setToolTipText(java.util.ResourceBundle
				.getBundle(TOOL_TIPS_BUNDLE).getString("quantizationStep"));
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 1;
		gridBagConstraints.gridy = 2;
		gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
		gridBagConstraints.weightx = 1.0;
		jPanel3.add(quantizationStepCombo, gridBagConstraints);

		jLabel11.setText("Quantization type: ");
		jLabel11.setToolTipText(java.util.ResourceBundle.getBundle(
				TOOL_TIPS_BUNDLE).getString("quantizationType"));
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 1;
		gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
		jPanel3.add(jLabel11, gridBagConstraints);

		quantizationTypeCombo.setModel(new javax.swing.DefaultComboBoxModel(
				new String[] { "expounded", "derived" }));
		quantizationTypeCombo.setToolTipText(java.util.ResourceBundle
				.getBundle(TOOL_TIPS_BUNDLE).getString("quantizationType"));
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 1;
		gridBagConstraints.gridy = 1;
		gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
		gridBagConstraints.weightx = 1.0;
		jPanel3.add(quantizationTypeCombo, gridBagConstraints);

		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 2;
		gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
		gridBagConstraints.weightx = 1.0;
		getContentPane().add(jPanel3, gridBagConstraints);

		jPanel4.setLayout(new java.awt.GridBagLayout());

		jPanel4.setBorder(javax.swing.BorderFactory
				.createTitledBorder("Rate allocation"));
		jLabel2.setText("Desired bit rate: ");
		jLabel2.setToolTipText(java.util.ResourceBundle.getBundle(
				TOOL_TIPS_BUNDLE).getString("bitRate"));
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 1;
		gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
		jPanel4.add(jLabel2, gridBagConstraints);

		bitRateField.setText("24.00");
		bitRateField.setToolTipText(java.util.ResourceBundle.getBundle(
				TOOL_TIPS_BUNDLE).getString("bitRate"));
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 1;
		gridBagConstraints.gridy = 1;
		gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
		gridBagConstraints.weightx = 1.0;
		jPanel4.add(bitRateField, gridBagConstraints);

		jLabel1.setText("Progression type: ");
		jLabel1.setToolTipText(java.util.ResourceBundle.getBundle(
				TOOL_TIPS_BUNDLE).getString("progressionType"));
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
		jPanel4.add(jLabel1, gridBagConstraints);

		progressionTypeCombo
				.setModel(new javax.swing.DefaultComboBoxModel(new String[] {
						"Resolution", "Layer", "RPCL", "PCRL", "CPRL" }));
		progressionTypeCombo.setSelectedIndex(1);
		progressionTypeCombo.setToolTipText(java.util.ResourceBundle.getBundle(
				TOOL_TIPS_BUNDLE).getString("progressionType"));
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
		gridBagConstraints.weightx = 1.0;
		jPanel4.add(progressionTypeCombo, gridBagConstraints);

		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 3;
		gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
		gridBagConstraints.weightx = 1.0;
		getContentPane().add(jPanel4, gridBagConstraints);

		pack();
	}// </editor-fold>

	public void openExternalFile(java.awt.event.ActionEvent evt) {
		// Handle open button action.

		// shows dialog
		int returnVal = jFileChooser.showOpenDialog(ControllInternalFrame.this);

		// file opening
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			File file = jFileChooser.getSelectedFile();
			externalImageField.setText(file.getAbsolutePath());
		}
	}

	private void modeChanged(java.awt.event.ActionEvent evt) {
		final boolean isLossy = modeCombo.getSelectedIndex() == 0;

		quantizationTypeCombo.setEnabled(isLossy);
		jLabel11.setEnabled(isLossy);
		quantizationStepCombo.setEnabled(isLossy);
		jLabel5.setEnabled(isLossy);
		bitRateField.setEnabled(isLossy);
		jLabel2.setEnabled(isLossy);
	}

	public void setExternalSourceEnabled(boolean enabled) {
		imageTypeCombo.setSelectedIndex(enabled ? 1 : 0);
		internalImageCombo.setEnabled(!enabled);
		jLabel8.setEnabled(!enabled);
		externalImageField.setEnabled(enabled);
		externalImageButton.setEnabled(enabled);
		jLabel9.setEnabled(enabled);
	}

	private void refresh(java.awt.event.ActionEvent evt) {
		EncoderOptions options = new EncoderOptions();
		options.setLossless(modeCombo.getSelectedIndex() == 1);
		options.setDecompositionLevels(decompositionLevelsCombo
				.getSelectedIndex());
		options.setGuardBits(guardBitsCombo.getSelectedIndex() + 1);
		options.setQuantizationType(quantizationTypeCombo.getSelectedIndex());
		options.setRescalePalette(rescaleCombo.getSelectedIndex() == 1);
		try {
			options.setQuantizationStep(quantizationStepCombo.getText());
		} catch (NumberFormatException ex) {
			JOptionPane.showMessageDialog(main,
					"Invalid value for 'rate' attribute");
			return;
		}
		options.setProgressionType(progressionTypeCombo.getSelectedIndex());
		try {
			options.setBitRate(bitRateField.getText());
		} catch (NumberFormatException ex) {
			JOptionPane.showMessageDialog(main,
					"Invalid value for 'step' attribute");
			return;
		}

		BufferedImage image = getImage();
		if (image == null)
			JOptionPane.showMessageDialog(main,
					"Couldn't access the file or bad file format");
		else if (image.getRaster().getNumBands() != 3)
			JOptionPane
					.showMessageDialog(main, "Only RGB images are supported");
		else
			main.setImage(image, options);

	}

	private BufferedImage getImage() {
		if (imageTypeCombo.getSelectedIndex() == 0) {
			// use classloader
			String imageName;
			if (internalImageCombo.getSelectedIndex() < 2) {
				imageName = internalImageCombo.getSelectedItem().toString()
						+ ".jpg";
			} else {

				imageName = internalImageCombo.getSelectedItem().toString()
						+ ".bmp";
			}

			System.out.print("Image name: " + imageName);
			try {
				return ImageIO.read(new MemoryCacheImageInputStream(getClass()
						.getClassLoader().getResourceAsStream(imageName)));
			} catch (IOException e) {
				return null;
			}
		} else {
			// use file io
			try {
				return ImageIO.read(new FileImageInputStream(new File(
						externalImageField.getText())));
			} catch (FileNotFoundException e) {
				return null;
			} catch (IOException e) {
				return null;
			}
		}
	}

	private void imageSourceChanged(java.awt.event.ActionEvent evt) {
		final boolean builtin = imageTypeCombo.getSelectedIndex() == 0;
		setExternalSourceEnabled(!builtin);
	}

	// Variables declaration - do not modify
	private javax.swing.JTextField bitRateField;

	private javax.swing.JComboBox decompositionLevelsCombo;

	private javax.swing.JButton externalImageButton;

	private javax.swing.JTextField externalImageField;

	private javax.swing.JComboBox guardBitsCombo;

	private javax.swing.JComboBox imageTypeCombo;

	private javax.swing.JComboBox internalImageCombo;

	private javax.swing.JLabel jLabel1;

	private javax.swing.JLabel jLabel10;

	private javax.swing.JLabel jLabel11;

	private javax.swing.JLabel jLabel2;

	private javax.swing.JLabel jLabel3;

	private javax.swing.JLabel jLabel4;

	private javax.swing.JLabel jLabel5;

	private javax.swing.JLabel jLabel6;

	private javax.swing.JLabel jLabel7;

	private javax.swing.JLabel jLabel8;

	private javax.swing.JLabel jLabel9;

	private javax.swing.JPanel jPanel1;

	private javax.swing.JPanel jPanel2;

	private javax.swing.JPanel jPanel3;

	private javax.swing.JPanel jPanel4;

	private javax.swing.JComboBox modeCombo;

	private javax.swing.JComboBox progressionTypeCombo;

	private javax.swing.JTextField quantizationStepCombo;

	private javax.swing.JComboBox quantizationTypeCombo;

	private javax.swing.JButton refreshButton;

	private javax.swing.JComboBox rescaleCombo;
	// End of variables declaration
}
