package pl.edu.agh.ssm2006.jpeg2000demo;

import java.awt.Image;
import java.awt.image.BufferedImage;
import java.awt.image.Raster;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.StreamTokenizer;
import java.io.StringReader;
import java.util.NoSuchElementException;
import java.util.StringTokenizer;

import jj2000.disp.BlkImgDataSrcImageProducer;
import jj2000.j2k.decoder.Decoder;
import jj2000.j2k.encoder.Encoder;
import jj2000.j2k.encoder.EncoderSpecs;
import jj2000.j2k.image.BlkImgDataSrc;
import jj2000.j2k.image.DataBlk;
import jj2000.j2k.image.ImgDataConverter;
import jj2000.j2k.image.Tiler;
import jj2000.j2k.image.forwcomptransf.ForwCompTransf;
import jj2000.j2k.image.input.ImgReader;
import jj2000.j2k.image.input.ImgReaderPPM;
import jj2000.j2k.quantization.quantizer.CBlkQuantDataSrcEnc;
import jj2000.j2k.quantization.quantizer.Quantizer;
import jj2000.j2k.util.FacilityManager;
import jj2000.j2k.util.MsgLogger;
import jj2000.j2k.util.ParameterList;
import jj2000.j2k.wavelet.analysis.CBlkWTData;
import jj2000.j2k.wavelet.analysis.CBlkWTDataSrc;
import jj2000.j2k.wavelet.analysis.ForwardWT;
import jj2000.j2k.wavelet.analysis.SubbandAn;
import pl.edu.agh.ssm2006.jpeg2000demo.io.ImgReaderImage;

public class EncoderDecoder {
	public static ParameterList defaultEncoderParams = new ParameterList();

	public static ParameterList defaultDecoderParams = new ParameterList();
	static {
		String[][] encoderParams = Encoder.getAllParameters();
		String[][] decoderParams = Decoder.getAllParameters();
		int i;

		for (i = encoderParams.length - 1; i >= 0; i--) {
			if (encoderParams[i][3] != null) {
				defaultEncoderParams.put(encoderParams[i][0],
						encoderParams[i][3]);
			}
		}

		for (i = decoderParams.length - 1; i >= 0; i--) {
			if (decoderParams[i][3] != null) {
				defaultDecoderParams.put(decoderParams[i][0],
						decoderParams[i][3]);
			}
		}
	}

	private Image image;

	private BufferedImage beforeQuantizationLuma;

	private BufferedImage beforeQuantizationChromaA;

	private BufferedImage beforeQuantizationChromaB;

	private BufferedImage afterQuantizationLuma;

	private BufferedImage afterQuantizationChromaA;

	private BufferedImage afterQuantizationChromaB;

	private File encodedFile;

	private ParameterList encoderParams = new ParameterList(
			defaultEncoderParams);

	private float qStep;

	private double rate;

	private boolean rescalePalette;

	public EncoderDecoder(BufferedImage image, EncoderOptions options) {
		this.image = image;
		this.qStep = options.getQuantizationStep();
		rescalePalette = options.isRescalePalette();
		options.populateParameterList(encoderParams);

		beforeQuantizationLuma = new BufferedImage(image.getWidth(null), image
				.getHeight(null), BufferedImage.TYPE_BYTE_GRAY);
		beforeQuantizationChromaA = new BufferedImage(image.getWidth(null),
				image.getHeight(null), BufferedImage.TYPE_BYTE_GRAY);
		beforeQuantizationChromaB = new BufferedImage(image.getWidth(null),
				image.getHeight(null), BufferedImage.TYPE_BYTE_GRAY);
		afterQuantizationLuma = new BufferedImage(image.getWidth(null), image
				.getHeight(null), BufferedImage.TYPE_BYTE_GRAY);
		afterQuantizationChromaA = new BufferedImage(image.getWidth(null),
				image.getHeight(null), BufferedImage.TYPE_BYTE_GRAY);
		afterQuantizationChromaB = new BufferedImage(image.getWidth(null),
				image.getHeight(null), BufferedImage.TYPE_BYTE_GRAY);

	}

	public void encode() {
		Raster raster = ((BufferedImage) image).getData();
		int height = raster.getHeight();
		int width = raster.getWidth();
		File rawFile;

		try {
			String header = "P6\n" + width + " " + height + "\n255\n";

			rawFile = File.createTempFile("decoded", ".ppm");
			rawFile.deleteOnExit();
			BufferedOutputStream bos = new BufferedOutputStream(
					new FileOutputStream(rawFile));
			bos.write(header.getBytes());
			for (int i = 0; i < height; i++) {
				for (int j = 0; j < width; j++) {
					bos.write(raster.getSample(j, i, 0));
					bos.write(raster.getSample(j, i, 1));
					bos.write(raster.getSample(j, i, 2));
				}
			}
			bos.close();

			encodedFile = File.createTempFile("encoded", ".j2k");
			encodedFile.deleteOnExit();
			
			ParameterList pl = encoderParams;

			encodeVisualisationImages(rawFile.getPath());

			// Input file
			pl.put("i", rawFile.getPath());

			// Output file
			pl.put("o", encodedFile.getPath());

			Encoder encoder = new Encoder(pl);

			encoder.run();

			rate = encodedFile.length() * 8d / width / height;
		} catch (IOException ex) {
			throw new RuntimeException(
					"IO exception while accessing temporary files");
		}
	}

	public void decode() {
		ParameterList pl = new ParameterList(defaultDecoderParams);
		pl.put("i", encodedFile.getPath());

		String type = "ppm";

		File decodedFile = null;
		try {
			decodedFile = decodedFile = File.createTempFile("decoded", "."
					+ type);
			decodedFile.deleteOnExit();
			
			pl.put("o", decodedFile.getPath());

			Decoder dec = new Decoder(pl);
			dec.run();

			ImgReaderPPM imr = new ImgReaderPPM(decodedFile.getPath());
			image = BlkImgDataSrcImageProducer.createImage(imr);
		} catch (IOException ex) {
			throw new RuntimeException(
					"IO exception while accessing temporary files");
		}
	}

	public void encodeVisualisationImages(String path) {
		boolean useFileFormat = false;
		StreamTokenizer stok;
		StringTokenizer sgtok;
		int ncomp = 3;
		boolean imsigned[] = new boolean[3];
		BlkImgDataSrc imgsrc;
		int i;
		int tw, th;
		int refx, refy;
		int trefx, trefy;
		int pktspertp;
		Tiler imgtiler;
		ForwCompTransf fctransf;
		ImgDataConverter converter;
		EncoderSpecs encSpec;
		ForwardWT dwt;
		Quantizer quant;
		String outname;
		float rate;

		ParameterList pl = encoderParams;
		pl.put("i", path);

		try {
			outname = pl.getParameter("o");

			if (pl.getParameter("file_format").equals("on")) {
				useFileFormat = true;
			}

			if (useFileFormat) {
				String outext = null;
				String outns = outname;
				if (outname.lastIndexOf('.') != -1) {
					outext = outname.substring(outname.lastIndexOf('.'),
							outname.length());
					outns = outname.substring(0, outname.lastIndexOf('.'));
				}

				if (outext == null || !outext.equalsIgnoreCase(".jp2")) {
					if (!pl.getBooleanParameter("disable_jp2_extension")) {
						FacilityManager.getMsgLogger().printmsg(
								MsgLogger.INFO,
								"JPEG 2000 file names"
										+ " end with .jp2 extension when using"
										+ " the file format of part 1. This "
										+ "extension is automatically"
										+ " added by JJ2000. Use "
										+ "'-disable_jp2_extension' to "
										+ "disable it.");

						outname = outns + ".jp2";
					}
				}
			}

			if (pl.getParameter("pph_tile").equals("on")) {
				if (pl.getParameter("Psop").equals("off")) {
					pl.put("Psop", "on");
				}
				if (pl.getParameter("Peph").equals("off")) {
					pl.put("Peph", "on");
				}
			}

			if (pl.getParameter("pph_main").equals("on")) {
				if (pl.getParameter("Psop").equals("off")) {
					pl.put("Psop", "on");
				}
				if (pl.getParameter("Peph").equals("off")) {
					pl.put("Peph", "on");
				}
			}

			try {
				rate = pl.getFloatParameter("rate");
				if (rate == -1) {
					rate = Float.MAX_VALUE;
				}
			} catch (NumberFormatException e) {
				// already validated
			}
			try {
				pktspertp = pl.getIntParameter("tile_parts");
				if (pktspertp != 0) {
					if (pl.getParameter("Psop").equals("off")) {
						pl.put("Psop", "on");
					}
					if (pl.getParameter("Peph").equals("off")) {
						pl.put("Peph", "on");
					}
				}
			} catch (NumberFormatException e) {
				e.printStackTrace();
				return;
			}

			// **** Tiler ****
			// get nominal tile dimensions
			stok = new StreamTokenizer(new StringReader(pl
					.getParameter("tiles")));
			stok.eolIsSignificant(false);

			stok.nextToken();
			if (stok.ttype != StreamTokenizer.TT_NUMBER) {
				return;
			}
			tw = (int) stok.nval;
			stok.nextToken();
			if (stok.ttype != StreamTokenizer.TT_NUMBER) {
				return;
			}
			th = (int) stok.nval;

			// Get image reference point
			sgtok = new StringTokenizer(pl.getParameter("ref"));
			try {
				refx = Integer.parseInt(sgtok.nextToken());
				refy = Integer.parseInt(sgtok.nextToken());
			} catch (NoSuchElementException e) {
				throw new IllegalArgumentException("Error while parsing 'ref' "
						+ "option");
			} catch (NumberFormatException e) {
				throw new IllegalArgumentException("Invalid number type in "
						+ "'ref' option");
			}
			if (refx < 0 || refy < 0) {
				throw new IllegalArgumentException("Invalid value in 'ref' "
						+ "option ");
			}

			// Get tiling reference point
			sgtok = new StringTokenizer(pl.getParameter("tref"));
			try {
				trefx = Integer.parseInt(sgtok.nextToken());
				trefy = Integer.parseInt(sgtok.nextToken());
			} catch (NoSuchElementException e) {
				throw new IllegalArgumentException(
						"Error while parsing 'tref' " + "option");
			} catch (NumberFormatException e) {
				throw new IllegalArgumentException("Invalid number type in "
						+ "'tref' option");
			}
			if (trefx < 0 || trefy < 0 || trefx > refx || trefy > refy) {
				throw new IllegalArgumentException("Invalid value in 'tref' "
						+ "option ");
			}

			imgsrc = new ImgReaderImage((BufferedImage) image);
			for (i = 0; i < ncomp; i++) {
				imsigned[i] = ((ImgReader) imgsrc).isOrigSigned(i);
			}

			// Instantiate tiler
			try {
				imgtiler = new Tiler(imgsrc, refx, refy, trefx, trefy, tw, th);
			} catch (IllegalArgumentException e) {
				if (pl.getParameter("debug").equals("on")) {
					e.printStackTrace();
				}
				return;
			}
			int ntiles = imgtiler.getNumTiles();

			// **** Encoder specifications ****
			encSpec = new EncoderSpecs(ntiles, 3, imgsrc, pl);

			// **** Component transformation ****
			if (pl.getParameter("Mct") != null
					&& pl.getParameter("Mct").equals("off")) {
				FacilityManager
						.getMsgLogger()
						.printmsg(
								MsgLogger.WARNING,
								"Input image is RGB and no color transform has "
										+ "been specified. Compression performance and "
										+ "image quality might be greatly degraded. Use "
										+ "the 'Mct' option to specify a color transform");
			}
			try {
				fctransf = new ForwCompTransf(imgtiler, encSpec);
			} catch (IllegalArgumentException e) {
				if (pl.getParameter("debug").equals("on")) {
					e.printStackTrace();
				}
				return;
			}

			// **** ImgDataConverter ****
			converter = new ImgDataConverter(fctransf);

			// **** ForwardWT ****
			try {
				dwt = ForwardWT.createInstance(converter, pl, encSpec);
			} catch (IllegalArgumentException e) {
				if (pl.getParameter("debug").equals("on")) {
					e.printStackTrace();
				}
				return;
			}

			dwt.setTile(0, 0);

			populateImage(dwt, 0, beforeQuantizationLuma);
			populateImage(dwt, 1, beforeQuantizationChromaA);
			populateImage(dwt, 2, beforeQuantizationChromaB);

			// **** Quantizer ****
			try {
				quant = Quantizer.createInstance(dwt, encSpec);
			} catch (IllegalArgumentException e) {
				if (pl.getParameter("debug").equals("on")) {
					e.printStackTrace();
				}
				return;
			}

			quant.setTile(0, 0);

			populateImage(quant, 0, afterQuantizationLuma);
			populateImage(quant, 1, afterQuantizationChromaA);
			populateImage(quant, 2, afterQuantizationChromaB);

		} catch (IllegalArgumentException e) {
			if (pl.getParameter("debug").equals("on")) {
				e.printStackTrace();
			}
			return;

		} catch (Error e) {
			if (pl.getParameter("debug").equals("on")) {
				e.printStackTrace();
			}
		} catch (RuntimeException e) {
			e.printStackTrace();
		} catch (Throwable e) {
			if (pl.getParameter("debug").equals("on")) {
				e.printStackTrace();
			}
			return;
		}
	}

	private void populateImage(final CBlkQuantDataSrcEnc src, final int comp,
			BufferedImage image) {
		populateImage(new ImageDataSource() {

			public CBlkWTData getData() {
				return src.getNextCodeBlock(comp, null);
			}

		}, image, true);
	}

	private void populateImage(final CBlkWTDataSrc src, final int comp,
			BufferedImage image) {
		populateImage(new ImageDataSource() {

			public CBlkWTData getData() {
				return src.getNextCodeBlock(comp, null);
			}

		}, image, false);
	}

	private void populateImage(ImageDataSource src, BufferedImage image,
			boolean afterQuantization) {
		CBlkWTData data;
		double[][] imageData = new double[image.getWidth()][image.getHeight()];

		double min = Float.MAX_VALUE, max = Float.MIN_VALUE;

		SubbandAn subband = null;
		for (;;) {
			data = src.getData();
			if (subband == null)
				subband = data.sb;
			if (data == null || subband != data.sb) {

				// rescale image palette
				if (rescalePalette || !afterQuantization) {
					for (int x = 0; x < image.getWidth(); x++)
						for (int y = 0; y < image.getHeight(); y++) {
							if (imageData[x][y] != Float.MIN_VALUE) {
								image
										.getRaster()
										.setSample(
												x,
												y,
												0,
												rescalePalette ? (imageData[x][y] - min)
														* (255 / (max - min))
														: imageData[x][y] + 128);
							}
						}
				}

				if (data == null)
					break;

				if (rescalePalette) {
					min = Float.MAX_VALUE;
					max = Float.MIN_VALUE;
					for (int x = 0; x < image.getWidth(); x++)
						for (int y = 0; y < image.getHeight(); y++)
							imageData[x][y] = Float.MIN_VALUE;
				}

				subband = data.sb;
			}

			for (int x = 0; x < data.w; x++) {
				for (int y = 0; y < data.h; y++) {
					double value;
					if (data.getDataType() == DataBlk.TYPE_FLOAT)
						value = ((float[]) data.getData())[x + y * data.w];
					else {
						value = ((int[]) data.getData())[x + y * data.w];
					}
					if (afterQuantization) {
						int intValue = (int) value;
						double step = qStep
								/ (data.sb.l2Norm * (1 << data.sb.anGainExp));
						int shiftBits = 31 - data.magbits;
						step /= (1 << shiftBits);

						intValue = (intValue >> shiftBits) << shiftBits;

						value = ((float) ((intValue >= 0) ? intValue
								: -(intValue & 0x7FFFFFFF)))
								* step;
					}
					imageData[data.ulx + x][data.uly + y] = value;

					min = Math.min(value, min);
					max = Math.max(value, max);
				}
			}
		}

		if (!rescalePalette && afterQuantization)
			for (int x = 0; x < image.getWidth(); x++)
				for (int y = 0; y < image.getHeight(); y++) {
					image.getRaster().setSample(
							x,
							y,
							0,
							imageData[x][y]
									* (127 / Math.max(Math.abs(max), Math
											.abs(min))) + 128);
				}
	}

	/**
	 * @return Returns the image.
	 */
	public Image getImage() {
		return image;
	}

	public Image getAfterQuantizationChromaA() {
		return afterQuantizationChromaA;
	}

	public Image getAfterQuantizationChromaB() {
		return afterQuantizationChromaB;
	}

	public Image getAfterQuantizationLuma() {
		return afterQuantizationLuma;
	}

	public Image getBeforeQuantizationChromaA() {
		return beforeQuantizationChromaA;
	}

	public Image getBeforeQuantizationChromaB() {
		return beforeQuantizationChromaB;
	}

	public Image getBeforeQuantizationLuma() {
		return beforeQuantizationLuma;
	}

	/**
	 * @return Returns the rate.
	 */
	public double getRate() {
		return rate;
	}
}
