package pl.edu.agh.ssm2006.jpeg2000demo;

import jj2000.j2k.util.ParameterList;

public class EncoderOptions {
	public static final int QIANTIZATION_TYPE_EXPOUNDED = 0;

	public static final int QIANTIZATION_TYPE_DERIVED = 1;

	public static final int PROGRESSION_TYPE_RESOLUTION = 0;

	public static final int PROGRESSION_TYPE_LAYER = 1;

	public static final int PROGRESSION_TYPE_RPCL = 2;

	public static final int PROGRESSION_TYPE_PCRL = 3;

	public static final int PROGRESSION_TYPE_CPRL = 4;

	private boolean lossless;

	private int decompositionLevels;

	private int guardBits;

	private int quantizationType;

	private float quantizationStep;

	private int progressionType;

	private float bitRate;
	
	private boolean rescalePalette;

	/**
	 * @return Returns the rescalePalette.
	 */
	public boolean isRescalePalette() {
		return rescalePalette;
	}

	/**
	 * @param rescalePalette The rescalePalette to set.
	 */
	public void setRescalePalette(boolean rescalePalette) {
		this.rescalePalette = rescalePalette;
	}

	public void populateParameterList(ParameterList params) {
		if (lossless) {
			params.put("lossless", "on");
		} else {
			params.put("lossless", "off");
			params.put("rate", bitRate + "");
			params.put("Qtype",
					quantizationType == QIANTIZATION_TYPE_DERIVED ? "derived"
							: "expounded");
		}

		params.put("Wlev", decompositionLevels + "");
		params.put("Qstep", quantizationStep + "");
		params.put("Qguard_bits", guardBits + "");
		switch (progressionType) {
		case PROGRESSION_TYPE_RESOLUTION:
			params.put("Aptype", "res");
			break;
		case PROGRESSION_TYPE_LAYER:
			params.put("Aptype", "layer");
			break;
		case PROGRESSION_TYPE_RPCL:
			params.put("Aptype", "res-pos");
			break;
		case PROGRESSION_TYPE_PCRL:
			params.put("Aptype", "pos-comp");
			break;
		default:
			params.put("Aptype", "comp-pos");
		}
	}

	/**
	 * @return Returns the bitRate.
	 */
	public float getBitRate() {
		return bitRate;
	}

	/**
	 * @param bitRate
	 *            The bitRate to set.
	 */
	public void setBitRate(String bitRate) throws NumberFormatException {
		this.bitRate = Float.parseFloat(bitRate);
		if (this.bitRate <= 0)
			throw new NumberFormatException();
	}

	/**
	 * @return Returns the decompositionLevels.
	 */
	public int getDecompositionLevels() {
		return decompositionLevels;
	}

	/**
	 * @param decompositionLevels
	 *            The decompositionLevels to set.
	 */
	public void setDecompositionLevels(int decompositionLevels) {
		this.decompositionLevels = decompositionLevels;
	}

	/**
	 * @return Returns the guardBits.
	 */
	public int getGuardBits() {
		return guardBits;
	}

	/**
	 * @param guardBits
	 *            The guardBits to set.
	 */
	public void setGuardBits(int guardBits) {
		this.guardBits = guardBits;
	}

	/**
	 * @return Returns the lossless.
	 */
	public boolean isLossless() {
		return lossless;
	}

	/**
	 * @param lossless
	 *            The lossless to set.
	 */
	public void setLossless(boolean lossless) {
		this.lossless = lossless;
	}

	/**
	 * @return Returns the progressionType.
	 */
	public int getProgressionType() {
		return progressionType;
	}

	/**
	 * @param progressionType
	 *            The progressionType to set.
	 */
	public void setProgressionType(int progressionType) {
		this.progressionType = progressionType;
	}

	/**
	 * @return Returns the quantizationStep.
	 */
	public float getQuantizationStep() {
		return quantizationStep;
	}

	/**
	 * @param quantizationStep
	 *            The quantizationStep to set.
	 */
	public void setQuantizationStep(String quantizationStep)
			throws NumberFormatException {
		this.quantizationStep = Float.parseFloat(quantizationStep);
	}

	/**
	 * @return Returns the quantizationType.
	 */
	public int getQuantizationType() {
		return quantizationType;
	}

	/**
	 * @param quantizationType
	 *            The quantizationType to set.
	 */
	public void setQuantizationType(int quantizationType) {
		this.quantizationType = quantizationType;
	}
}
