package pl.edu.agh.ssm2006.jpeg2000demo;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.File;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JDesktopPane;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.SwingConstants;

public class Main extends JFrame implements ActionListener {
	JDesktopPane desktop;

	// Menu items
	private JMenuItem openFileMenuItem = new JMenuItem("Open...");

	private JCheckBoxMenuItem controlsMenuItem = new JCheckBoxMenuItem(
			"Controls");

	private JCheckBoxMenuItem origAndEncodedMenuItem = new JCheckBoxMenuItem(
			"Original and encoded image");

	private JCheckBoxMenuItem qLumaMenuItem = new JCheckBoxMenuItem(
			"Luminance compare");

	private JCheckBoxMenuItem qChromaAMenuItem = new JCheckBoxMenuItem(
			"Chrominance A compare");

	private JCheckBoxMenuItem qChromaBMenuItem = new JCheckBoxMenuItem(
			"Chrominance B compare");

	// frames
	private ControllInternalFrame controls = new ControllInternalFrame(
			"Controls", this);

	private ImageInternalFrame origAndEncodedImage = new ImageInternalFrame(
			"Original end encoded image comparison", origAndEncodedMenuItem);

	private ImageInternalFrame qLumaImage = new ImageInternalFrame(
			"Luminance wavelet before and after quantization", qLumaMenuItem);

	private ImageInternalFrame qChromaAImage = new ImageInternalFrame(
			"Chrominance A wavelet before and after quantization",
			qChromaAMenuItem);

	private ImageInternalFrame qChromaBImage = new ImageInternalFrame(
			"Chrominance B wavelet before and after quantization",
			qChromaBMenuItem);

	private JLabel statusBar = new JLabel();

	private final static String SPACES = "                      ";

	public Main() {
		super("JPEG 2000 Demo");

		JFrame.setDefaultLookAndFeelDecorated(true);

		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		setBounds(0, 0, screenSize.width, screenSize.height - 50);
		// Set up the GUI.
		desktop = new JDesktopPane(); // a specialized layered pane
		setContentPane(desktop);// Create and set up the window.

		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		JMenuBar menuBar = createMenuBar();
		setJMenuBar(menuBar);
		statusBar.setHorizontalAlignment(SwingConstants.RIGHT);
		menuBar.add(statusBar);
		statusBar.setFont(new Font("Monospaced", Font.PLAIN, statusBar
				.getFont().getSize()));
		statusBar.setText(SPACES + "Status: ready");
		setVisible(true);
		createFrames();
	}

	protected JMenuBar createMenuBar() {
		JMenuBar menuBar = new JMenuBar();

		// Set up the lone menu.
		JMenu menu = new JMenu("File");
		menuBar.add(menu);

		openFileMenuItem.setActionCommand("open");
		openFileMenuItem.addActionListener(this);
		menu.add(openFileMenuItem);

		JMenuItem menuItem = new JMenuItem("Quit");
		menuItem.setActionCommand("quit");
		menuItem.addActionListener(this);
		menu.add(menuItem);

		menu = new JMenu("View");
		menuBar.add(menu);

		controlsMenuItem.setSelected(true);
		controlsMenuItem.setActionCommand("controls");
		controlsMenuItem.addActionListener(this);
		menu.add(controlsMenuItem);

		origAndEncodedMenuItem.setSelected(true);
		origAndEncodedMenuItem.setActionCommand("origenc");
		origAndEncodedMenuItem.addActionListener(this);
		menu.add(origAndEncodedMenuItem);

		qLumaMenuItem.setActionCommand("qLuma");
		qLumaMenuItem.addActionListener(this);
		menu.add(qLumaMenuItem);

		qChromaAMenuItem.setActionCommand("qChromaA");
		qChromaAMenuItem.addActionListener(this);
		menu.add(qChromaAMenuItem);

		qChromaBMenuItem.setActionCommand("qChromaB");
		qChromaBMenuItem.addActionListener(this);
		menu.add(qChromaBMenuItem);

		menu = new JMenu("Help");
		menuItem = new JMenuItem("About");
		menuItem.setActionCommand("about");
		menuItem.addActionListener(this);
		menu.add(menuItem);
		menuBar.add(menu);

		return menuBar;
	}

	protected void createFrames() {
		controls.setLocation(desktop.getWidth() - controls.getWidth(), desktop
				.getHeight()
				- controls.getHeight());
		controls.setVisible(true);
		desktop.add(controls);

		origAndEncodedImage.setLocation(0, 0);
		origAndEncodedImage.setVisible(true);

		desktop.add(origAndEncodedImage);
		desktop.add(qLumaImage);
		desktop.add(qChromaAImage);
		desktop.add(qChromaBImage);
	}

	public static void main(String[] args) {
		// Schedule a job for the event-dispatching thread:
		// creating and showing this application's GUI.
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				new Main();
			}
		});
	}

	public void setImage(final BufferedImage image, final EncoderOptions options) {
		statusBar.setText(SPACES + "Status: calculating...");
		controls.disableRefreshButton();

		new Thread() {
			public void run() {
				// places for decoded image
				Image decodedImage;
				Image beforeQLuma, afterQLuma;
				Image beforeQChromaA, afterQChromaA;
				Image beforeQChromaB, afterQChromaB;

				// start of time measuring
				long begin = System.currentTimeMillis();

				// create encoder/decoder with given parameters
				EncoderDecoder encDec = new EncoderDecoder(image, options);
				// encode, decode and get images
				encDec.encode();
				encDec.decode();
				decodedImage = encDec.getImage();
				beforeQLuma = encDec.getBeforeQuantizationLuma();
				afterQLuma = encDec.getAfterQuantizationLuma();
				beforeQChromaA = encDec.getBeforeQuantizationChromaA();
				afterQChromaA = encDec.getAfterQuantizationChromaA();
				beforeQChromaB = encDec.getBeforeQuantizationChromaB();
				afterQChromaB = encDec.getAfterQuantizationChromaB();

				// end of time measuring
				long end = System.currentTimeMillis();

				// extraction of image size
				int width = ((BufferedImage) image).getWidth();
				int height = ((BufferedImage) image).getHeight();

				// original and encoded image construction and show
				// construction of place for images - with 5 black pixels
				// between images
				BufferedImage origAndEncodedBufferedImage = new BufferedImage(
						width * 2 + 5, height, BufferedImage.TYPE_INT_RGB);
				// get graphic context
				Graphics2D origAndEncodedGraphics2 = (Graphics2D) origAndEncodedBufferedImage
						.getGraphics();
				// left and right image draw
				origAndEncodedGraphics2.drawImage(((BufferedImage) image),
						null, 0, 0);
				origAndEncodedGraphics2.drawImage(decodedImage, width + 5, 0,
						null);
				// image is set
				origAndEncodedImage.setImage(origAndEncodedBufferedImage);

				// construction of luma comparison image
				// construction of place for images - with 5 black pixels
				// between images
				BufferedImage qLumaBufferedImage = new BufferedImage(
						width * 2 + 5, height, BufferedImage.TYPE_INT_RGB);
				// get graphic context
				Graphics2D qLumaGraphics2 = (Graphics2D) qLumaBufferedImage
						.getGraphics();
				// left and right image draw
				qLumaGraphics2.drawImage(beforeQLuma, 0, 0, null);
				qLumaGraphics2.drawImage(afterQLuma, width + 5, 0, null);
				// image is set
				qLumaImage.setImage(qLumaBufferedImage);

				// construction of ChromaA comparison image
				// construction of place for images - with 5 black pixels
				// between images
				BufferedImage qChromaABufferedImage = new BufferedImage(
						width * 2 + 5, height, BufferedImage.TYPE_INT_RGB);
				// get graphic context
				Graphics2D qChromaAGraphics2 = (Graphics2D) qChromaABufferedImage
						.getGraphics();
				// left and right image draw
				qChromaAGraphics2.drawImage(beforeQChromaA, 0, 0, null);
				qChromaAGraphics2.drawImage(afterQChromaA, width + 5, 0, null);
				// image is set
				qChromaAImage.setImage(qChromaABufferedImage);

				// construction of ChromaA comparison image
				// construction of place for images - with 5 black pixels
				// between images
				BufferedImage qChromaBBufferedImage = new BufferedImage(
						width * 2 + 5, height, BufferedImage.TYPE_USHORT_GRAY);
				// get graphic context
				Graphics2D qChromaBGraphics2 = (Graphics2D) qChromaBBufferedImage
						.getGraphics();
				// left and right image draw
				qChromaBGraphics2.drawImage(beforeQChromaB, 0, 0, null);
				qChromaBGraphics2.drawImage(afterQChromaB, width + 5, 0, null);
				// image is set
				qChromaBImage.setImage(qChromaBBufferedImage);

				controls.enableRefreshButton();

				// elapsed time calculation and display
				long time = end - begin;
				statusBar.setText(SPACES + "Status: calculation time: "
						+ (time / 1000.) + "s, achieved bit-rate: "
						+ String.format("%1$.3f", encDec.getRate()));
			}
		}.start();
	}

	public void actionPerformed(ActionEvent e) {
		if (e.getActionCommand().equals("quit"))
			System.exit(0);
		else if (e.getActionCommand().equals("open")) {
			controls.openExternalFile(null);

			controls.setExternalSourceEnabled(true);
		} else if (e.getActionCommand().equals("controls"))
			controls.setVisible(!controls.isVisible());
		else if (e.getActionCommand().equals("origenc"))
			origAndEncodedImage.setVisible(!origAndEncodedImage.isVisible());
		else if (e.getActionCommand().equals("qLuma"))
			qLumaImage.setVisible(!qLumaImage.isVisible());
		else if (e.getActionCommand().equals("qChromaA"))
			qChromaAImage.setVisible(!qChromaAImage.isVisible());
		else if (e.getActionCommand().equals("qChromaB"))
			qChromaBImage.setVisible(!qChromaBImage.isVisible());
		else if (e.getActionCommand().equals("about"))
			JOptionPane
					.showMessageDialog(
							this,
							"JPEG 2000 Demo\nby\nJakub Madej\nBart\u0142omiej \u0141abno\n\nThis software uses JJ2000 library");
	}

	/**
	 * @return Returns the controlsMenuItem.
	 */
	public JCheckBoxMenuItem getControlsMenuItem() {
		return controlsMenuItem;
	}
}
