/**
 * 
 */
package pl.edu.agh.ssm2006.jpeg2000demo.io;

import java.awt.image.BufferedImage;

import jj2000.j2k.image.DataBlk;
import jj2000.j2k.image.DataBlkInt;
import jj2000.j2k.image.input.ImgReader;

/**
 * 
 * 
 */
public class ImgReaderImage extends ImgReader {
	private BufferedImage image;

	private int[][] barr = new int[3][];

	private DataBlkInt intBlk;

	private DataBlkInt dbi = new DataBlkInt();

	public ImgReaderImage(BufferedImage image) {
		this.image = image;
		h = image.getHeight(null);
		w = image.getWidth(null);
		nc = 3;
	}

	@Override
	public void close() {
	}

	@Override
	public boolean isOrigSigned(int c) {
		if (c < 0 || c > 2)
			throw new IllegalArgumentException();
		return false;
	}

	public DataBlk getCompData(DataBlk blk, int c) {
		// NOTE: can not directly call getInterCompData since that returns
		// internally buffered data.
		int w, h;

		// Check type of block provided as an argument
		if (blk.getDataType() != DataBlk.TYPE_INT) {
			DataBlkInt tmp = new DataBlkInt(blk.ulx, blk.uly, blk.w, blk.h);
			blk = tmp;
		}

		int bakarr[] = (int[]) blk.getData();
		// Save requested block size
		w = blk.w;
		h = blk.h;
		// Force internal data buffer to be different from external
		blk.setData(null);
		getInternCompData(blk, c);
		// Copy the data
		if (bakarr == null) {
			bakarr = new int[w * h];
		}
		if (blk.offset == 0 && blk.scanw == w) {
			// Requested and returned block buffer are the same size
			System.arraycopy(blk.getData(), 0, bakarr, 0, w * h);
		} else { // Requested and returned block are different
			for (int i = h - 1; i >= 0; i--) { // copy line by line
				System.arraycopy(blk.getData(), blk.offset + i * blk.scanw,
						bakarr, i * w, w);
			}
		}
		blk.setData(bakarr);
		blk.offset = 0;
		blk.scanw = blk.w;
		return blk;
	}

	public DataBlk getInternCompData(DataBlk blk, int c) {
		// Check component index
		if (c < 0 || c > 2)
			throw new IllegalArgumentException();

		// Check type of block provided as an argument
		if (blk.getDataType() != DataBlk.TYPE_INT) {
			if (intBlk == null)
				intBlk = new DataBlkInt(blk.ulx, blk.uly, blk.w, blk.h);
			else {
				intBlk.ulx = blk.ulx;
				intBlk.uly = blk.uly;
				intBlk.w = blk.w;
				intBlk.h = blk.h;
			}
			blk = intBlk;
		}

		// If asking a component for the first time for this block, read the 3
		// components
		if ((barr[c] == null) || (dbi.ulx > blk.ulx) || (dbi.uly > blk.uly)
				|| (dbi.ulx + dbi.w < blk.ulx + blk.w)
				|| (dbi.uly + dbi.h < blk.uly + blk.h)) {
			int i, mi;

			// Reset data arrays if needed
			if (barr[c] == null || barr[c].length < blk.w * blk.h) {
				barr[c] = new int[blk.w * blk.h];
			}
			blk.setData(barr[c]);

			i = (c + 1) % 3;
			if (barr[i] == null || barr[i].length < blk.w * blk.h) {
				barr[i] = new int[blk.w * blk.h];
			}
			i = (c + 2) % 3;
			if (barr[i] == null || barr[i].length < blk.w * blk.h) {
				barr[i] = new int[blk.w * blk.h];
			}

			// set attributes of the DataBlk used for buffering
			dbi.ulx = blk.ulx;
			dbi.uly = blk.uly;
			dbi.w = blk.w;
			dbi.h = blk.h;

			mi = blk.uly + blk.h;
			for (i = blk.uly; i < mi; i++) {
				for (int cnt = 0; cnt < blk.w; cnt++) {
					for (int comp = 0; comp < 3; comp++)
						barr[comp][(i-blk.uly) * blk.w + cnt] = image.getRaster()
								.getSample(blk.ulx + cnt, i, comp)-128;
				}
			}

			// Set buffer attributes
			blk.setData(barr[c]);
			blk.offset = 0;
			blk.scanw = blk.w;
		} else { // Asking for the 2nd or 3rd block component
			blk.setData(barr[c]);
			blk.offset = (blk.ulx - dbi.ulx) * dbi.w + blk.ulx - dbi.ulx;
			blk.scanw = dbi.scanw;
		}

		// Turn off the progressive attribute
		blk.progressive = false;
		return blk;
	}

	public int getFixedPoint(int c) {
		if (c < 0 || c > 2)
			throw new IllegalArgumentException();
		return 0;
	}

	public int getNomRangeBits(int c) {
		if (c < 0 || c > 2)
			throw new IllegalArgumentException();

		return 8;
	}

}
